{% extends "multiseek/base.html" %}
{% load i18n %}

{% block content %}
    {% if description %}
        {% trans "Query: " %}{{ description|safe }}
    {% endif %}

    {% for element in object_list %}
        <li>{{ element }}</li>
    {% empty %}
        {% trans "No elements" %}
    {% endfor %}


{% endblock %}