{% load i18n %}

{% block report_type %}
    {% if report_types %}
        <table>
            <tr>
                <td>
                    {% trans "Report type: " %}</td>
                <td>
                    <select name="{{ MULTISEEK_REPORT_TYPE }}">
                        {% for t in report_types %}
                            <option value="{{ forloop.counter0 }}">{{ t.label }}</option>
                        {% endfor %}
                    </select></td>
            </tr>
        </table>
        <script type="text/javascript">
            /* Update results window with change of report type */
            $("select[name={{ MULTISEEK_REPORT_TYPE }}]").change(function () {
                $("#sendQueryButton").click();
            });
        </script>
    {% endif %}
{% endblock %}
